#!/usr/bin/env python
# coding: utf-8

import math

def get_star_points(start_theta, num_points, skip, bbox):
    '''Generate the points for a star centered in the bounding box.'''
    # bbox = (xmin, ymin, xmax, ymax)
    wid = bbox[2] - bbox[0]
    hgt = bbox[3] - bbox[1]
    rx = wid / 2
    ry = hgt / 2
    cx = bbox[0] + rx
    cy = bbox[1] + ry

    points = []

    # Make sure skip is between 0 and num_points - 1.
    skip = skip % num_points

    # Is skip > num_points / 2, use the smaller equivalent.
    if skip > num_points / 2:
        skip = num_points - skip

    # Do not allow skip = 0.
    if skip == 0:
        raise ValueError('skip should not be 0 or a multiple of num_points')

    # If this is a polygon, don't bother with concave points.
    if skip == 1:
        theta = start_theta
        dtheta = 2 * math.pi / num_points
        for i in range(num_points):
            point = (cx + rx * math.cos(theta),
                     cy + ry * math.sin(theta))
            points.append(point)
            theta += dtheta
        return points

    # Find the radius for the concave vertices.
    inner_fraction = calculate_inner_radius(num_points, skip)

    # Make the points.
    theta = start_theta
    dtheta = math.pi / num_points
    for i in range(num_points):
        points.append((
            cx + rx * math.cos(theta),
            cy + ry * math.sin(theta)))
        theta += dtheta
        points.append((
            cx + rx * math.cos(theta) * inner_fraction,
            cy + ry * math.sin(theta) * inner_fraction))
        theta += dtheta
    return points

def calculate_inner_radius(num_points, skip):
    '''Calculate the inner star radius.'''
    # If skip = num_points / 2, make a fake star instead of an asterisk.
    if math.isclose(skip, num_points / 2):
        return 0.33

    # Calculate angles to key points.
    dtheta = 2 * math.pi / num_points
    theta00 = -math.pi / 2
    theta01 = theta00 + dtheta * skip
    theta10 = theta00 + dtheta
    theta11 = theta10 - dtheta * skip

    # Find the key points.
    pt00 = (math.cos(theta00), math.sin(theta00))
    pt01 = (math.cos(theta01), math.sin(theta01))
    pt10 = (math.cos(theta10), math.sin(theta10))
    pt11 = (math.cos(theta11), math.sin(theta11))

    # See where the segments connecting the points intersect.
    lines_intersect, segments_intersect, intersection, close_p1, close_p2 = \
        find_line_intersection(pt00, pt01, pt10, pt11)

    # Calculate the distance between the point of intersection and the center.
    return math.sqrt(
        intersection[0] * intersection[0] +
        intersection[1] * intersection[1])

def find_line_intersection(p1, p2, p3, p4):
    '''
    Find the point of intersection between the lines p1 --> p2 and p3 --> p4.
    Return:
        lines_intersect - True if the lines intersect
        segments_intersect - True if the segments intersect
        intersection - The point of lines' intersection
        close_p1 - The point on line 1 closest to the intersection
        close_p2 - The point on line 2 closest to the intersection
    '''
    # Get the segments' parameters.
    dx12 = p2[0] - p1[0]
    dy12 = p2[1] - p1[1]
    dx34 = p4[0] - p3[0]
    dy34 = p4[1] - p3[1]

    # Solve for t1 and t2
    denominator = dy12 * dx34 - dx12 * dy34

    try:
        t1 = ((p1[0] - p3[0]) * dy34 + (p3[1] - p1[1]) * dx34) / denominator
    except ZeroDivisionError:
        # The lines are parallel (or close enough to it).
        lines_intersect = False
        segments_intersect = False
        intersection = None
        close_p1 = None
        close_p2 = None
        return lines_intersect, segments_intersect, intersection, \
            close_p1, close_p2
    except Exception as ex:
        print(f'Unexpected error, {type(ex).__name__}: {ex}')
        raise

    lines_intersect = True

    t2 = ((p3[0] - p1[0]) * dy12 + (p1[1] - p3[1]) * dx12) / -denominator

    # Find the point of intersection.
    intersection = (p1[0] + dx12 * t1, p1[1] + dy12 * t1)

    # The segments intersect if t1 and t2 are between 0 and 1.
    segments_intersect = ((t1 >= 0) and (t1 <= 1) and (t2 >= 0) and (t2 <= 1))

    # Find the closest points on the segments.
    if t1 < 0:
        t1 = 0
    elif t1 > 1:
        t1 = 1

    if t2 < 0:
        t2 = 0
    elif t2 > 1:
        t2 = 1

    close_p1 = (p1[0] + dx12 * t1, p1[1] + dy12 * t1)
    close_p2 = (p3[0] + dx34 * t2, p3[1] + dy34 * t2)

    return lines_intersect, segments_intersect, intersection, close_p1, close_p2

# In[18]:
import tkinter as tk
from tkinter import messagebox

class LSystemsApp:
    def __init__(self):
        self.window = tk.Tk()
        self.window.title('draw_stars')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('400x400')

        # Build the UI.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, anchor=tk.W, padx=3, pady=3)

        label = tk.Label(frame, text='# Points:')
        label.pack(side=tk.LEFT, padx=3)
        self.num_points_var = tk.StringVar(value='9')
        num_points_entry = tk.Entry(frame, textvariable=self.num_points_var, width=3)
        num_points_entry.pack(side=tk.LEFT, padx=3)

        label = tk.Label(frame, text='Skip:')
        label.pack(side=tk.LEFT, padx=3)
        self.skip_var = tk.StringVar(value='4')
        skip_entry = tk.Entry(frame, textvariable=self.skip_var, width=3)
        skip_entry.pack(side=tk.LEFT, padx=3)

        button = tk.Button(frame, text='Draw', width=10, command=self.draw)
        button.pack(side=tk.LEFT, padx=10)

        self.canvas = tk.Canvas(self.window, bg='white',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # If the user presses Enter, draw.
        self.window.bind('<Return>', lambda event: self.draw())

        # Display the window.
        num_points_entry.focus_force()
        self.window.mainloop()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

    def draw(self):
        # Delete any previous drawing objects.
        self.canvas.delete(tk.ALL)

        # Get the drawing parameters.
        try:
            num_points = int(self.num_points_var.get())
        except:
            num_points = 0
        if num_points < 3:
            messagebox.showerror('Invalid Input',
                '# Points must be an integer greater than 2.')
            return

        try:
            skip = int(self.skip_var.get())
        except:
            skip = 0
        if skip % num_points == 0:
            messagebox.showerror('Invalid Input',
                'Skip must be an integer that\'s not 0 or a multiple of the number of points.')
            return

        # Get the canvas's bounding  box.
        wid = self.canvas.winfo_width() - 1
        hgt = self.canvas.winfo_height() - 1
        radius = min(wid, hgt) / 2 - 10
        cx = wid / 2
        cy = hgt / 2
        bbox = (cx - radius, cy - radius, cx + radius, cy + radius)

        # Get the points.
        points = get_star_points(-math.pi / 2, num_points, skip, bbox)

        # Draw the star.
        self.canvas.delete(tk.ALL)
        self.canvas.create_polygon(points, fill='', outline='blue')

        # Draw the bounding box.
        # self.canvas.create_rectangle(bbox, fill='', outline='red')

# In[19]:
LSystemsApp()
print('Done')
